<?php

/**
 * @file
 * Administration page for Domain Override module.
 */

/**
 * Administration form.
 */
function domain_override_settings_form(&$form_state) {
  $form['domain_override_types'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Overridable content types'),
    '#default_value' => _domain_override_overridable_types(),
    '#description' => t('The content types selected will be overridable on a per-domain basis by users with the matching "override ___ content" permission.'),
  );
  foreach (node_get_types('names') as $type => $name) {
    $form['domain_override_types']['#options'][$type] = check_plain($name);
  }
  $form['buttons']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save configuration'),
  );
  $form['#theme'] = 'system_settings_form';
  return $form;
}

/**
 * Submit callback. Stores settings.
 */
function domain_override_settings_form_submit($form, &$form_state) {
  foreach ($form_state['values']['domain_override_types'] as $type => $value) {
    variable_set('domain_override_type_'. $type, $value !== 0);
  }
}

/**
 * Form-building callback to safely confirm the overriding of the node.
 */
function domain_override_override_confirm(&$form_state, $node) {
  /**
   * Check to see if this is the default translation.
   */
  // Only perform this check if there are translations.
  if($node->tnid) {
    // store the original language name
    $original_language = db_result(db_query("SELECT {name} FROM {languages} WHERE language='%s'", $node->language));
    // find the "master" node for this node's tnid, where the nid equals the tnid
    $result = db_result(db_query("SELECT nid FROM {node} where tnid=%d AND tnid=nid", array(':tnid' => $node->tnid)));
    if ($result != $node->nid) {
      $node = node_load($result);
      $default_language = db_result(db_query("SELECT name FROM {languages} WHERE language='%s'", $node->language));
      // warn the user that they are actually overriding the default translation for the node, but copying over all its translations.
      drupal_set_message(t("You've chosen to override the %original_language translation of this node, which is different from the default translation for this node (%default_language). In order to maintain compatability, all translations of this node will be overridden. You will be taken to the overridden %default_language node's edit screen.", array('%original_language' => $original_language, '%default_language' => $default_language)), 'error');
    }
  }
  // Build the form using the proper node
  $form['domain_override_node'] = array(
    '#type' => 'value',
    '#value' => $node,
  );
  global $_domain;
  $question = t('Are you sure you want to override this node for the %current_domain domain?', array('%current_domain' => $_domain['sitename']));
  $path = 'node/' . $node->nid;
  $description = t("This action cannot be undone. <strong><em>This node's page will become unavailable to view or edit in the current domain.</em></strong> All menu links in the current domain that pointed to this page will now point to the new node. You will be taken to the new, cloned node's edit form to make the necessary changes.");
  $yes = t('Override');
  $no = t('Cancel');
  return confirm_form($form, $question, $path, $description, $yes, $no);
}

/**
 * Submit handler function for the domain_override_override_confirm form.
 */
function domain_override_override_confirm_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    $node = $form_state['values']['domain_override_node'];
    domain_override_override_page($node);
  }
}

/**
 * Processing function for confirmed node/%node/override.
 */
function domain_override_override_page($node) {
  global $_domain, $user;

  // Clone the node (with inspiration from the node_clone project)
  $original_node = $node;
  // @TODO: The original node should no longer be published in the default domain.
  // Need to find a way to unpublish it without removing access to its menu link in
  // the overridden domain. Leaving it published for now.
  // unpublish the original node for all domains (so its menu item remains active)
  //_domain_override_unpublish_domains($original_node, $_domain['domain_id']);

  $node = drupal_clone($node);
  $node->nid = NULL;
  $node->vid = NULL;
  $node->tnid = NULL;
  $node->name = $user->name;
  $node->uid = $user->uid;
  $node->created = NULL;
  $node->book['mlid'] = NULL;
  $node->path = NULL;
  $node->files = array();
  $node->title = t('Override for !domain: !title', array('!title' => $node->title, '!domain' => $_domain['subdomain']));
  // New node should be accessible to just one domain.
  _domain_override_set_single_domain($node, $_domain['domain_id']);
  // Save the new node the first time.
  node_save($node);

  // Copy/override translations of original node, too
  if (_domain_override_copy_translations($original_node, $node, $_domain['domain_id']) > 0) {
    // We have at least one translation, so we must set source node's tnid.
    $node->tnid = $node->nid;
  }
  // Allow other modules to alter the finalized new node, and save again.
  drupal_alter('domain_override_node', $node, $original_node);
  node_save($node);

  // Keep track of the override.
  $domain_override = new stdClass;
  $domain_override->nid = $original_node->nid;
  $domain_override->nid_override = $node->nid;
  $domain_override->domain_id = $_domain['domain_id'];
  drupal_write_record('domain_override', $domain_override);

  // Send user to edit their new node.
  drupal_goto("node/{$node->nid}/edit");
}

/**
 * Copies and domain-overrides translations of $original_node as
 * translations of $node. Copies only translations that are currently published
 * to the target domain.
 *
 * @param $original_node
 *   Node object of source node; its translations should be copied.
 * @param $node
 *   Node object of "target" node; the new translations will be marked as
 *   translations of this node.
 * @param $domain_id
 *   Numerical domain id of the domain to which the new translation nodes should
 *   be published.
 * @return
 *   The number of translations that were copied.
 */
function _domain_override_copy_translations($original_node, $node, $domain_id) {
  assert(is_object($original_node) && is_object($node));
  assert(is_numeric($domain_id));
  GLOBAL $_domain, $user;

  if (!module_exists('translation')) {
    return 0;
  }
  $translations_copied = 0;
  $translations = translation_node_get_translations($original_node->nid);
  foreach ($translations as $translation_node) {
    // Original node is listed as a translation of itself; skip it.
    if ($translation_node->nid == $original_node->nid) {
      continue;
    }

    // translation_node_get_translations() gives partial nodes; load the rest.
    $translation_node = node_load($translation_node->nid);
    $original_translation_nid = $translation_node->nid;
    // Only copy/override translations that are published to the target domain.
    if (!($translation_node->domain_site || in_array($domain_id, $translation_node->domains))) {
      continue;
    }
    // @TODO: The original node should no longer be published in the default domain.
    // Need to find a way to unpublish it without removing access to its menu link in
    // the overridden domain. Leaving it published for now.
    // activates the original translation for all domains (so its menu item remains active)
    // _domain_override_unpublish_domains($translation_node, $domain_id);

    // Copy the translation, using $node's domain publishing settings.
    $translation_node->nid = NULL;
    $translation_node->vid = NULL;
    $translation_node->uid = $user->uid;
    $translation_node->tnid = $node->nid;
    $translation_node->created = NULL;
    $translation_node->book['mlid'] = NULL;
    $translation_node->files = array();
    $translation_node->path = NULL;
    _domain_override_set_single_domain($translation_node, $domain_id);
    node_save($translation_node);

    $translations_copied++;
  }
  return $translations_copied;
}

/**
 * Sets a node to be published on only one domain.
 *
 * @param $node
 *   Node for which to set publishing domain.
 * @param $domain_id
 *   Target domain (numeric id) to set as the node published domain.
 */
function _domain_override_set_single_domain(&$node, $domain_id) {
  assert(is_object($node));

  // gather domains so we can look up target domain's sitename
  $domains = domain_domains();
  assert(is_array($domains[$domain_id]));

  $node->domains = array($domain_id => $domain_id);
  $node->domain_site = FALSE; // All affiliates? No.
  $node->subdomains = array($domain_id => $domains[$domain_id]['sitename']);
}

/**
 * Unpublishes a node from the given domain.
 *
 * @param $node
 *   Node object of node to unpublish.
 * @param $domain_id
 *   Numerical domain id of the domain from which to unpublish the node.
 */
/* @TODO: make this work with the domain access setting.
 * Disabled because the original menu items should stay active for all domains.
 * Original menu items stay active and point to original nodes,
 * even when the node is overridden. (We intercept them based on the original nid.)
function _domain_override_unpublish_domains(&$node, $domain_id) {
  assert($node->domain_site || isset($node->domains[$domain_id]));

  // If node was published to all affiliates, we must make sure the individual
  // domains are all listed in $node->domains and remove "publish for all"
  if ($node->domain_site) {
    $node->domains = drupal_map_assoc(array_keys(domain_domains()));
    $node->domain_site = FALSE;
    // we just rebuilt domains list, and id 0 (main domain) must be stored as
    // -1 there.
    if (isset($node->domains[0])) {
      unset($node->domains[0]);
      $node->domains[-1] = -1;
    }
  }
  // Unpublish $node from the given domain
  //unset($node->domains[$domain_id]);
  node_save($node);
}
*/
